""" 
    
    An adapter for PloneFormGen that saves submitted form data
    to Salesforce.com
    
"""

__author__  = ''
__docformat__ = 'plaintext'

# Python imorts
import logging

# Zope imports
from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent
from zope.interface import providedBy

# Plone imports
from Products.Archetypes.public import StringField, SelectionWidget, \
    DisplayList, Schema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import registerATCT
from Products.CMFCore.permissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

# DataGridField
from Products.DataGridField import DataGridField, DataGridWidget
from Products.DataGridField.SelectColumn import SelectColumn
from Products.DataGridField.FixedColumn import FixedColumn
from Products.DataGridField.DataGridField import FixedRow

# Interfaces
from Products.PloneFormGen.interfaces import IPloneFormGenField

# PloneFormGen imports
from Products.PloneFormGen.content.actionAdapter import \
    FormActionAdapter, FormAdapterSchema

# Local imports
from Products.salesforcepfgadapter.config import PROJECTNAME

logger = logging.getLogger("PloneFormGen")    

schema = FormAdapterSchema.copy() + Schema((
    StringField('SFObjectType',
        searchable=0,
        required=1,
        default=u'Contact',
        mutator='setSFObjectType',
        widget=SelectionWidget(
            label='Salesforce Object Type',
            i18n_domain = "ploneformgen",
            label_msgid = "label_salesforce_type_text",
            ),
        vocabulary='displaySFObjectTypes',
        ),
    DataGridField('fieldMap',
         searchable=0,
         required=1,
         schemata='field mapping',
         columns=('form_field', 'sf_field'),
         fixed_rows = "generateFormFieldRows",
         allow_delete = False,
         allow_insert = False,
         allow_reorder = False,
         widget = DataGridWidget(
             label='Form fields to Salesforce fields mapping',
             label_msgid = "label_salesforce_field_map",
             description="""The following Form Fields are available\
                 with your Form Folder. Choose the appropriate \
                 Salesforce Field for each Form Field.""",
             description_msgid = 'help_salesforce_field_map',
             columns= {
                 "form_field" : FixedColumn("Form Fields"),
                 "sf_field" : SelectColumn("Salesforce Fields", 
                                           vocabulary="buildSFFieldOptionList")
             },
             i18n_domain = "ploneformgen",
         ),
    )    
))

class SalesforcePFGAdapter(FormActionAdapter):
    """ An adapter for PloneFormGen that saves results to Salesforce.
    """
    
    schema=schema
    
    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
    
    meta_type = portal_type = 'SalesforcePFGAdapter'
    archetype_name = 'Salesforce PFG Adapter'
    content_icon   = 'salesforce.gif'
    
    def initializeArchetype(self, **kwargs):
        """Initialize Private instance variables
        """
        FormActionAdapter.initializeArchetype(self, **kwargs)
        
        # All Salesforce fields for the current Salesforce object type. Since
        # we need this for every row in our field mapping widget, it's better
        # to just set it on the object when we set the Salesforce object type. 
        # This way we don't query Salesforce for every field on our form.
        self._fieldsForSFObjectType = [] 
    
    security = ClassSecurityInfo()
    def onSuccess(self, fields, REQUEST=None):
        """ The essential method of a PloneFormGen Adapter:
        - collect the submitted form data
        - examine our field map to determine which Saleforce fields
          to populate
        - if there are any mappings, submit the data to Salesforce
          and check the result
        """
        logger.debug('Calling onSuccess()')
        sObject = dict(type=self.SFObjectType)
        for field in fields:
            formFieldName = field.Title()
            formFieldValue = REQUEST.form.get(field.fgField.getName(),'')
            if not self._getSFFieldForFormField(formFieldName):
                continue
            salesforceFieldName = self._getSFFieldForFormField(formFieldName)
            sObject[salesforceFieldName] = formFieldValue
        if len(sObject.keys()) > 1:
            salesforce = getToolByName(self, 'portal_salesforcebaseconnector')
            result = salesforce.create(sObject)[0]
            if result['success']:
                logger.debug("Successfully created new %s %s in Salesforce" % \
                             (self.SFObjectType, result['id']))
            else:
                errorStr = 'Failed to create new %s in Salesforce: %s' % \
                    (str(self.SFObjectType), result['errors'][0]['message'])
                raise errorStr
        else:
            logger.warn('No valid field mappings found. Not calling Salesforce.')
    
    def setSFObjectType(self, newType):
        """When we set the Salesforce object type,
           we also need to reset all the possible fields
           for our mapping selection menus.
        """
        logger.debug('Calling setSFObjectType()')
        self.SFObjectType = newType
        self._fieldsForSFObjectType = self._querySFFieldsForType()
    
    def displaySFObjectTypes(self):
        logger.debug('Calling displaySFObjectTypes()')        
        """ returns vocabulary for available Salesforce Object Types 
            we can create. 
        """
        types = self._querySFObjectTypes()
        typesDisplay = DisplayList()
        for type in types:
            msgId = 'vocabulary_'  + type.lower() + '_text'
            typesDisplay.add(type, self.translate(msgid=msgId,domain='ploneformgen',default=type))
        return typesDisplay
    
    def buildSFFieldOptionList(self):
        """Returns a DisplayList of all the fields
           for the currently selected Salesforce object
           type.
        """
        dl = DisplayList()
        dl.add('', 'None')
        fields = self._fieldsForSFObjectType
        fields.sort()
        for field in fields:
            dl.add(field, field)
        return dl
    
    def generateFormFieldRows(self):
        """This method returns a list of rows for the field mapping
           ui. One row is returned for each field in the form folder.
        """
        form_folder = aq_parent(self)
        form_field_titles = []
        
        for formField in form_folder.objectIds():
            fieldObj = getattr(form_folder, formField)
            if IPloneFormGenField.providedBy(fieldObj):
                logger.debug("creating mapper row for %s" % fieldObj)
                form_field_titles.append(FixedRow(keyColumn="form_field",
                        initialData={"form_field" : fieldObj.Title(), "sf_field" : ""}))
        return form_field_titles
    
    def _querySFFieldsForType(self):
        """Return a tuple of all the possible fields for the current
           Salesforce object type
        """
        SFObjectType = self.SFObjectType
        salesforce = getToolByName(self, 'portal_salesforcebaseconnector')
        salesforceFields = salesforce.describeSObjects(SFObjectType)[0].fields.keys()
        return salesforceFields
    
    def _querySFObjectTypes(self):
        """Returns a tuple of all Salesforce object type names.
        """
        salesforce = getToolByName(self, 'portal_salesforcebaseconnector')
        types = salesforce.describeGlobal()['types']
        return types
    
    def _getSFFieldForFormField(self, target):
        """  Return the Salesforce field
             mapped to a given Form field. 
        """
        sfField = None
        for mapping in self.fieldMap:
            if mapping['form_field'] == target and mapping['sf_field']:
                sfField = mapping['sf_field'] 
                break
        return sfField
    

registerATCT(SalesforcePFGAdapter, PROJECTNAME)
